import pytest

from run import PageDataTree, Tree


def test_simple_data():
  data = {"a": 1, "b": 2}
  with pytest.raises(StopIteration):
    assert next(PageDataTree(data).tree_by_key(data, 'c'))


def test_nested_simple_data():
  data = {"a": 1, "b": 2, "d": {"c": 3}}

  assert next(PageDataTree(data).tree_by_key(data, 'c')) == PageDataTree.join_tree(
    tree=PageDataTree.join_tree(Tree.ROOT, 'd'), key='c'
  )


def test_simple_data_with_list():
  data = {"a": 1, "b": 2, "d": [{"c": 3}]}

  assert next(PageDataTree(data).tree_by_key(data, 'c')) == PageDataTree.join_tree(
    tree=PageDataTree.join_tree(Tree.ROOT, 'd'),
    key='c',
    list_index=0
  )


def test_simple_data_with_none():
  data = {"a": 1, "b": None, None: 4, "d": [{"c": 3}]}

  assert next(PageDataTree(data).tree_by_key(data, 'c')) == PageDataTree.join_tree(
    tree=PageDataTree.join_tree(Tree.ROOT, 'd'),
    key='c',
    list_index=0
  )


def test_simple_data_for_noreturn():
  data = {"a": 1, "b": None, None: 4, "d": [{"c": 3}]}

  assert next(PageDataTree(data).tree_by_key(data, 'c', result_to='print')) is None


def test_simple_data_value_with_list_index():
  data = {"a": 1, "b": 2, "d": [{"c": 3}]}

  # 'root -> d -> [0] -> c'
  tree = PageDataTree.join_tree(
    tree=PageDataTree.join_tree(Tree.ROOT, 'd'),
    key='c',
    list_index=0
  )

  assert PageDataTree(data).data_by_tree(tree) == 3



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from typing import Optional
from pprint import pprint

from .config import FD, Tree, Key
from .utils import FileIO

from .run import PageDataTree


def main(
    input_filepath: str, output_filepath: str,
    key: Optional[str], tree: Optional[str], view: Optional[bool]
) -> Optional[NotImplementedError]:
  file_io = FileIO(input_filepath)
  file_data = file_io.load()

  pdt = PageDataTree(file_data)

  if key or view:
    pdt_tree = pdt.tree_by_key_or_view(
      key=key, view=view,
      result_to=(Key.SAVE if output_filepath else Key.SHOW)
    )
  elif tree:
    pdt_tree = (tree,)
  else:
    raise NotImplementedError("[k]ey or [t]ree or [v]iew is required")

  for tree in pdt_tree:
    if not tree:
      continue

    tree_data = pdt.data_by_tree(tree)

    if output_filepath:
      if output_filepath == FD.STDOUT:
        print(tree)
        pprint(tree_data)
        print()
        continue

      if key:
        raise NotImplementedError('Use `-t` flag to dump a value for `tree`')

      file_io.dump(tree_data, output_filepath)


if __name__ == '__main__':
  parser = argparse.ArgumentParser(
    description='CLI tool to view json tree / structure'
  )

  parser.add_argument(
    '-i',
    type=str,
    help='[i]nput filepath'
  )
  parser.add_argument(
    '-o',
    type=str,
    help='[o]utput filepath'
  )
  parser.add_argument(
    '-v',
    type=bool,
    nargs='?',
    const=True,
    help='[v]view json tree'
  )
  parser.add_argument(
    '-k',
    type=str,
    help='[k]ey to search'
  )
  parser.add_argument(
    '-t',
    type=str,
    help='[t]ree to `-o` the data from'
  )
  parser.add_argument(
    '-l',
    type=int,
    help='[l]imit stdout'
  )
  parser.add_argument(
    '-il',
    type=int,
    help='[i]tems [l]imit stdout'
  )
  parser.add_argument(
    '-fk',
    type=str,
    help='[f]ilter stdout by `key`'
  )
  parser.add_argument(
    '-fv',
    type=str,
    help='[f]ilter stdout by `value`'
  )
  args = parser.parse_args()

  input_filepath = args.i
  output_filepath = args.o
  key = args.k
  tree = args.t
  view = args.v

  if not input_filepath:
    print('[i]nput filepath is required')
    exit()
  if not any((key, tree, view)):
    print('[k]ey or [t]ree or [v]iew is required')
    exit()

  if args.l:
    # decrements the limit befor returning the `tree`
    Tree.SEARCH_LIMIT = (args.l+1)
  if args.il:
    Tree.SEARCH_ITEMS_LIMIT = args.il

  Tree.SEARCH_FILTER_KEY = args.fk or Tree.SEARCH_FILTER_KEY
  Tree.SEARCH_FILTER_VALUE = args.fv or Tree.SEARCH_FILTER_VALUE

  main(input_filepath, output_filepath, key, tree, view)




import os
import json
from typing import Optional, Union


class FileIO:
  """File Input/Output

  Arguments
  ---------
  str
    `src` (source), filepath to load

  Methods
  -------
  load - reads the given `json` filepath
  dump - writes the `data` to the given `dst` (destination)
  """

  def __init__(self, src: str):
    self.src = src

  @staticmethod
  def validate_path(path: str) -> Optional[FileNotFoundError]:
    if not os.path.exists(path):
      raise FileNotFoundError(path)

  def dump(self, data: dict, dst: str, indent: int = 4) -> None:
    with open(dst, 'w') as ftw:
      ftw.write(json.dumps(data, indent=indent))

  def load(self) -> Union[dict, FileNotFoundError]:
    self.validate_path(self.src)

    with open(self.src, 'r') as ftr:
      return json.loads(ftr.read())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from jsontp.__init__ import __version__

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


setup(
  name='jsontp',
  version=__version__,
  description='JSON Tree Parser',
  long_description=long_description,
  long_description_content_type='text/markdown',
  author='Nodaa Gaji',
  author_email='c0d3r.nodiru.gaji@gmail.com',
  url='https://pypi.org/project/jsontp',
  download_url='https://github.com/ames0k0/jsontp',
  packages=['jsontp'],
  license='License :: OSI Approved :: MIT License',
  classifiers=[
    'Programming Language :: Python :: 3.10',
  ],
  platforms=[
    'Operating System :: OS Independent',
  ],
)
